import {
  Loader, Metadata, IDictionary, I18n, Router, dayjs as moment,
} from '@zeedhi/core';
import {
  Form, DialogService, LoadingService, Select,
} from '@zeedhi/common';
import { getStoredData } from '@zeedhi/zeedhi-miragejs';
import { callMakeServer } from '../server/server';
import { getConfig } from '../server/config';
import { useMirage } from '../server/use-mirage';

export class ImportacaoController {
  private utilities = Loader.getInstance('Utilities');

  public unidades: IDictionary = this.utilities.getItemLocalStorage('Units') || [];

  public agrupamentosProdutos: IDictionary = this.utilities.getItemLocalStorage('GruposProduto') || [];

  public async importar() {
    const form = Metadata.getInstance<Form>('formImportacao');
    if (form.validate()) {
      if (form.value.UNIDADE.length <= 30) {
        if (form.value.INTERVAL.length === 2) {
          const { value } = form;
          if (this.utilities.validateInterval(value.INTERVAL[0], value.INTERVAL[1])) {
            if (localStorage.getItem('hasImportedData') !== 'true') {
              this.continueImportacao();
            } else this.hasImportedData();
          }
        } else this.utilities.showAlert('errorFilter', 'FILTER_2_DATES', 'error');
      } else this.utilities.showAlert('errorMaxFilial', 'ERROR_MAX_FILIAL', 'error');
    }
  }

  public continueImportacao() {
    this.utilities.validateActiveUser().then((valid: any) => {
      if (valid) {
        this.utilities.verifyVersionStatus().then((versionValid: any) => {
          if (versionValid) {
            this.executeImportacao();
          }
        });
      }
    });
  }

  public executeImportacao() {
    LoadingService.show(' ');
    const form = Metadata.getInstance<Form>('formImportacao');
    const { value } = form;
    const filialConcat = value.UNIDADE.map((u: any) => u.CDFILIAL).join('_');
    localStorage.setItem('FILIAIS_IMPORTACAO', filialConcat);
    const centralConcat = value.UNIDADE.map((u: any) => (u.CDFILCENTPME ? u.CDFILCENTPME : null)).join('_');
    localStorage.setItem('CENTRAIS_IMPORTACAO', centralConcat);
    const dataIni = moment(value.INTERVAL[0], 'YYYY-MM-DD').format('DD/MM/YYYY');
    localStorage.setItem('DATA_INICIAL', dataIni);
    const dataFin = moment(form.value.INTERVAL[1], 'YYYY-MM-DD').format('DD/MM/YYYY');
    localStorage.setItem('DATA_FINAL', dataFin);
    const cdgrupprod = value.AGRUPAMENTO_PRODUTO?.CDGRUPPROD ? value.AGRUPAMENTO_PRODUTO?.CDGRUPPROD : 'T';
    localStorage.setItem('CDGRUPPROD', cdgrupprod);
    const operador = localStorage.getItem('CDOPERADOR') || '';
    localStorage.setItem('CDOPERADOR_IMPORTACAO', operador);
    const serverConfig = getConfig(true);
    const dateLastSinc = moment().format(this.utilities.getFormatLocaleHour());
    callMakeServer(serverConfig).then(() => {
      setTimeout(() => {
        const cdFilialSelecionada = localStorage.getItem('USER_CDFILIALSELECIONADA');
        const select = Metadata.getInstance<Select>('CDFILIALLOGADA');
        select.datasource.get().then(() => {
          if (filialConcat.includes(cdFilialSelecionada)) {
            select.setValue(cdFilialSelecionada);
          } else {
            select.setValue(value.UNIDADE[0].CDFILIAL);
          }
        });
        getStoredData('produtos').then((result) => {
          let text = '';
          if (result) text = 'scm::SUCCESS_IMPORT';
          else text = 'scm::SUCCESS_IMPORT_NO_PRODUCTS';
          LoadingService.hide();
          DialogService.show({
            name: 'successImport',
            type: 'success',
            title: I18n.translate('scm::SUCCESS'),
            text: I18n.translate(text),
            buttons: [
              {
                name: 'ok',
                component: 'ZdButton',
                label: I18n.translate('scm::OK'),
                events: {
                  click: () => {
                    localStorage.setItem('hasImportedData', 'true');
                    localStorage.setItem('request-log', '[]');
                    // localStorage.setItem('request-error-log', '[]');
                    localStorage.setItem('dataUltimaSincronizacao', dateLastSinc);
                    useMirage(true);
                    DialogService.hide();
                    Router.push('/');
                  },
                },
              },
            ],
          });
        });
      }, 20000);
    });
  }

  public hasImportedData() {
    // this.utilities.showAlert('alreadyImported', 'HAS_IMPORTED_DATA', 'e')
    DialogService.show({
      name: 'confirmImport',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::HAS_IMPORTED_DATA'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: {
            click: () => {
              DialogService.hide();
              this.continueImportacao();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public get getFilterFiliais() {
    return {
      CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
      CDFILIAL: 'T',
    };
  }

  public checkAccess() {
    if (!navigator.onLine) {
      DialogService.show({
        name: 'errorImport',
        type: 'error',
        title: I18n.translate('scm::ERROR'),
        text: I18n.translate('scm::IMPORT_CONNECTION_FAIL'),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: I18n.translate('scm::OK'),
            events: {
              click: () => {
                DialogService.hide();
                Router.push('/');
              },
            },
          },
        ],
      });
    } else {
      useMirage(false);
    }
  }

  public beforeDestroy() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      useMirage(true);
    } else useMirage(false);
  }
}
